@@ -20,7 +20,8 @@
 posterior_probs <- function(model, log_space = FALSE){
   fb <- forward_backward(model, log_space = log_space)
   if (!log_space) {
-  fb$forward_probs * fb$backward_probs
+    fb$forward_probs * fb$backward_probs / 
+      aperm(replicate(sum(model$n_states), fb$scaling_factors), c(3, 1, 2))
   } else {
     ll <- logLik(model, partials = TRUE, log_space = TRUE)
     fb$forward_probs + fb$backward_probs - 
