@@ -1,9 +1,10 @@
 // Forward-backward algorithm for mixture hidden Markov models
-#include "seqHMM.h"
+#include "forward_backward.h"
+#include "reparma.h"
 
 // [[Rcpp::export]]
 
-List forwardbackwardx(const arma::mat& transition, const arma::cube& emission,
+Rcpp::List forwardbackwardx(const arma::mat& transition, const arma::cube& emission,
   const arma::vec& init, const arma::ucube obs, const arma::mat& coef, const arma::mat& X,
   const arma::uvec& numberOfStates, bool forwardonly, unsigned int threads) {
   
@@ -29,17 +30,17 @@ List forwardbackwardx(const arma::mat& transition, const arma::cube& emission,
   }
   
   if (forwardonly) {
-    return List::create(Named("forward_probs") = wrap(alpha),
-      Named("scaling_factors") = wrap(scales));
+    return Rcpp::List::create(Rcpp::Named("forward_probs") = Rcpp::wrap(alpha),
+      Rcpp::Named("scaling_factors") = Rcpp::wrap(scales));
   } else {
     arma::cube beta(emission.n_rows, obs.n_cols, obs.n_slices); //m,n,k
     internalBackwardx(sp_trans, emission, obs, beta, scales, threads);
     if(!beta.is_finite()) {
       Rcpp::stop("Backward probabilities contain non-finite values. Check the model or try using the log-space version of the algorithm.");
     }
-    return List::create(Named("forward_probs") = wrap(alpha), Named("backward_probs") = wrap(beta),
-      Named("scaling_factors") = wrap(scales));
+    return Rcpp::List::create(Rcpp::Named("forward_probs") = Rcpp::wrap(alpha), Rcpp::Named("backward_probs") = Rcpp::wrap(beta),
+      Rcpp::Named("scaling_factors") = Rcpp::wrap(scales));
   }
   
-  return wrap(alpha);
+  return Rcpp::wrap(alpha);
 }
